function [b,xc,er] = fastls(s,z,n);

% function [b,xc,er] = fastls(s,z,n);
% derives the instantaneous amplitudes for sinusoidal (sometimes damped)
% components, and decomposes into sines+residual.
% An estimation by a recursive least mean squares which benefits from
% the specific structure of the vector family.
% s : data
% z : z = [z1, z2, z3,...], complex poles
% n : analysis length
% b : instantaneous amplitudes, % for instance, for a component at
%     normalized frequency nu, z = exp(j*2*pi*nu).
% xc: sinusoidal part
% er: residual (s = er+xc);
% ref. [1] Fast sequential LS estimation for sinusoidal modeling 
% and decomposition of audio signals,  Bertrand David et Roland Badeau, 
% 2007 IEEE Workshop on Applications of Signal Processing 
% to Audio and Acoustics WASPAA2007, New Paltz, New York, 
% USA, 21-24 octobre 2007, pp. 211-214
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.


N = length(s);
Np = length(z); % poles number


xc = zeros(1,N);
er = zeros(1,N);
dm = zeros(length(z),N);
b = zeros(length(z),N);

% initialization

V = vdm(z,n);d = pinv(V)*s(1:n);dm(:,1)=d;b(:,1)=d;
R = V'*V; iR = pinv(R);q1 = iR*(1./conj(z)); q2 = iR*conj(z).^(n-1);
xc0 = sum(d);v = (z.^n).';xcn = v*d;e0 = s(1)-xc0;en = s(n+1)-xcn;
xc(1) = xc0;
er(1) = e0;

% main loop
for k = n+2:N
	tn = k-n-1;
   d0 = d;
	d = z.*d - e0*q1 + en*q2;
	dm(:,tn+1) = d;
   b(:,tn+1)=b(:,tn).*(d + eps)./z./(d0+eps);
	xc0 = sum(d);
	xcn = v*d;
	e0 = s(tn+1)-xc0;
   er(tn+1)=e0;
	en = s(tn+n+1)-xcn;    
   xc(tn+1)=xc0;
end



function V = vdm(z,N);

% function V = vdm(z,N);
% vandermonde (colonne)
% a partir des poles z pour un ordre N
% V =   | 1    1    ..    1    |
%       | z0   z1   ..    zk   |
%       |  ...                 |
%       | z0^(N-1) z1^(N-1) ..    zk^(N-1) |

z = z(:).';n=(0:N-1)';
V = exp( n*log(z) );
