function [poles, amplitudes] = HR_analysis(signal, order, whitening)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% [poles, amplitudes] = HR_analysis(signal, order, whitening)
%
% Signal modeling with exponentially damped sinusoids (EDS)
% Parameters estimation with the ESPRIT algorithm
% O.Derrien, LMA Marseille & R.Badeau, LTCI Paris - 2009                        
%
% signal : vector representing the audio signal
% order : model order i.e. number of poles and amplitudes
%         maximum value is length(signal)/4
% whitening : optional whitening filter before HR analysis
%             0 = whitening disabled
%             1 = whitening enabled
%             whitening usually favors high frequency components
% poles : vector of complex numbers representing damping and frequency
% amplitudes : vector of complex numbers representing magnitude and phase
%              magnitudes are normalized, i.e. represent the maximum
%              value of the exponential envelope
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initialization
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[rows, columns] = size(signal);
if rows < columns
   signal = signal.';
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Whitening filter
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if (whitening ~= 0)

   % FFT analysis
   N = length(signal);
   Np = 2^nextpow2(2*N-1);
   S = abs(fft(signal.*hanning(N), Np)).^2/N;

   % Spectrum smoothing
   q = 51;
   H = hankel([S(end-(q-1)/2+1 : end); S(1 : (q+1)/2)], [S((q+1)/2 : end); S(1 : (q-1)/2)]);
   H = sort(H, 1);
   S = H(q/3,:).';

   % Computation of the whitening filter
   p = 5;
   a = zeros(p, 1);
   e = zeros(p,1);
   e(1) = 1;
   rx = ifft(S);
   a = toeplitz(conj(rx(1 : p)), rx(1 : p)) \ e;
   sigma2 = a(1);
   a = a/sigma2;

   % application of the whitening filter
   x = filter(a, 1, signal);

else

   % no whitening
   x = signal;

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Eigenvalues of the correlation matrix
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

lines = ceil(length(x)/2)+1;
columns = floor(length(x)/2);
N = lines+columns-1;
H = hankel(x(1 : lines), x(lines : N));
[U,S,V] = svd(H);
W = U(:, 1 : order);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Estimation of poles
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

phi = pinv(W(1 : end-1, :)) * W(2 : end, :);
poles = eig(phi);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Estimation of amplitudes
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

V = zeros(N, order);
for k2=1 : order
   if abs(poles(k2)) < 1,
       V(:, k2) = poles(k2).^((0 : N-1)');
   else
       V(:, k2) = poles(k2).^((-(N-1) : 0)');
   end
   V(:, k2) = V(:, k2) / norm(V(:, k2));
end
amplitudes = pinv(V)*x;

% Correction of amplitudes when the whitening filter is used
if (whitening ~= 0)

   V = exp(log(1./poles)*(0 : p-1));
   amplitudes = amplitudes ./ (V*a);

end


