function [prototype, analysis, synthesis] = analytic_PQMF(taps, channels)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% [prototype, analysis, synthesis] = analytic_PQMF(taps, channels)
%
% Computation of a analytic PQMF filter-bank
% i.e. keeping only positive frequencies
% O.Derrien, LMA Marseille & R.Badeau / J.Prado, LTCI Paris - 2009                        
% Algorithm for optimizing the prototype filter by P.P.Vaidyanathan
%
% taps : length of the analysis/synthesis filters, must be even
% channels : number of channels in the filter-bank, must be even
% prototype : vector representing the coefficients of the prototype filter
% analysis : matrix representing the coefficients of the analysis filters
% synthesis : matrix representing the coefficients of the synthesis filters
%
% To avoid instabilities, the ratio taps/channels should be maximum 32 
%
% The original signal can be recovered after analysis/synthesis by taking
% twice the real part of the synthesized signal
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Computation of the low-pass prototype filter
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

ws = pi/channels;
alpha = 100;
grid = taps;
tau = 0.5;
q = zeros(taps/2, 1);
err = 1e-4;
d = ones(grid, 1);
inip = firls(taps-1, [0. 1/(2*channels)-1/(8*channels) 1/(2*channels)+1/(8*channels) 1.], [1 1 0 0]);
p = inip(1 : taps/2);
wp = (0 : pi/channels/(grid-1) : pi/channels)';
ind = [(taps-1)/2 : -1 : 1/2];
C1 = wp*ind;
CC1 = 2*cos(C1);
Us = zeros(taps/2);
pimws = (pi-ws)/2;
for i = 1 : taps/2,
	arg1 = 2*i-taps-1;
	for j = 1 : taps/2,
		if (i == j),
			Us(i,i) = 4*(pimws-sin(arg1*ws)/(2*arg1));
		else
			imj = i-j;
			jmi = -imj;
			ipj = i+j-taps-1;
			Us(i, j) = 4*(sin(imj*ws)/(2*jmi) - sin(ipj*ws)/(2*ipj));
		end
	end
end
dev = (p-q')*(p'-q);
iter = 0;
while (dev > err)
	iter = iter + 1;
	Mp = CC1*p';
	Hp = diag(Mp);
	U = Hp*CC1;
	U = U+flipud(U);
	q = inv(U'*U+alpha*Us)*(U'*d);
	p = p-tau*(p-q');
	dev = sqrt((p-q')*(p'-q));
end
prototype = [p fliplr(p)];
prototype = prototype';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Computation of cosine-modulated analysis filters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

n = ((0:length(prototype)-1)-(taps-1)/2)';
analysis = zeros(taps,channels);
j = sqrt(-1);
for k = 1 : channels
	analysis(:, k)= prototype.*exp(j*(2*(k-1)+1)*pi*(n/(2*channels)+0.25));
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Computation of synthesis filters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

synthesis=conj(flipud(analysis));

