function onset = onset_detection(signal, sensitivity)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% onset = onset_detection(signal)
% onset = onset_detection(signal, sensitivity)
%
% Onset detection with a hybrid constant-Q subband analysis                                 
% Based on the method by C.Duxbury et al. (DAFx'02)       
% O.Derrien, LMA Marseille - 2009                        
%                                                         
% signal : vector representing the audio signal
%          sample rate 44.1 or 48 kHz
% sensitivity : threshold constant for onset detection
%               0 = minimum sensitivity
%               1 = maximum sensitivity
%               default = 0.01
% onset : vector of onset positions, in samples
%
% The signal can have a different sample rate, but the results
% may be inaccurate
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Multi-resolution time-frequency analysis
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% signal length
[rows, columns] = size(signal);
if (rows < columns)
   signal = signal.';
   L = columns;
else
   L = rows;
end

% minimum FFT size
N = 256;

% 5th subband: Fs/4 -> Fs/2
M = N;
K = floor(L/M*2)-1;
TF5 = zeros(N/4, K);
for k = 1 : K
   x = signal((k-1)*M/2+1 : (k+1)*M/2);
   X = fft(x.*hanning(M));
   TF5(:, k) = X(N/4+1 : N/2);
end

% 4th subband: Fs/8 -> Fs/4
M = 2*M;
K = floor(L/M*2)-1;
TF4 = zeros(N/4, K);
for k = 1 : K
   x = signal((k-1)*M/2+1 : (k+1)*M/2);
   X = fft(x.*hanning(M));
   TF4(:, k) = X(N/4+1 : N/2);
end

% 3rd subband: Fs/16 -> Fs/8
M = 2*M;
K = floor(L/M*2)-1;
TF3 = zeros(N/4, K);
for k = 1 : K
   x = signal((k-1)*M/2+1 : (k+1)*M/2);
   X = fft(x.*hanning(M));
   TF3(:, k) = X(N/4+1 : N/2);
end

% 1st and 2nd subband: 0 -> Fs/32, Fs/32 -> Fs/16
M = 2*M;
K = floor(L/M*2)-1;
TF2 = zeros(N/4, K);
TF1 = zeros(N/4, K);
TF12 = zeros(N/2, K);
for k = 1 : K
   x = signal((k-1)*M/2+1 : (k+1)*M/2);
   X = fft(x.*hanning(M));
   TF2(:, k) = X(N/4+1 : N/2);
   TF1(:, k) = X(1 : N/4);
   TF12(:, k) = X(1 : N/2);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Jumps in energy, subbands 1 -> 5 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Transient selection threshold
T_tr = 0.1;

% Regularization constant
epsilon = 1e-10;

% 5th subband: Fs/4 -> Fs/2
K = size(TF5, 2);
SE5 = max(sum(abs(TF5.^2)), epsilon);
JE5 = zeros(K, 1);
SE_prev = epsilon;
for k = 1 : K
   JE5(k) = log(SE5(k) / SE_prev);
   SE_prev = SE5(k);
end

% 4th subband: Fs/8 -> Fs/4
M = 2*M;
K = size(TF4, 2);
SE4 = max(sum(abs(TF4.^2)), epsilon);
JE4 = zeros(K, 1);
SE_prev = epsilon;
for k = 1 : K
   JE4(k) = log(SE4(k) / SE_prev);
   SE_prev = SE4(k);
end

% 3th subband: Fs/16 -> Fs/8
% transient energy measure
M = 2*M;
K = size(TF3, 2);
SE3 = zeros(K, 1);
X1 = zeros(N/4, 1);
X2 = zeros(N/4, 1);
JE3 = zeros(K, 1);
SE_prev = epsilon;
for k = 1 : K
   X0 = TF3(:, k);
   t = angle(X2) - 2*angle(X1) + angle(X0);
   tr_bins = find((angle(X2) - 2*angle(X1) + angle(X0)) < T_tr);
   SE = max(sum(abs(X0(tr_bins)).^2), epsilon);
   JE3(k) = log(SE / SE_prev);
   X2 = X1;
   X1 = X0;
   SE_prev = SE;
end

% 2th subband: Fs/32 -> Fs/16
% transient energy measure
M = 2*M;
K = size(TF2, 2);
SE2 = zeros(K, 1);
X1 = zeros(N/4, 1);
X2 = zeros(N/4, 1);
JE2 = zeros(K, 1);
SE_prev = epsilon;
for k = 1 : K
   X0 = TF2(:, k);
   t = angle(X2) - 2*angle(X1) + angle(X0);
   tr_bins = find((angle(X2) - 2*angle(X1) + angle(X0)) < T_tr);
   SE = max(sum(abs(X0(tr_bins)).^2), epsilon);
   JE2(k) = log(SE / SE_prev);
   X2 = X1;
   X1 = X0;
   SE_prev = SE;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Distance measure, subbands 1 -> 2 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

K = size(TF12, 2);
DM12 = zeros(K, 1);
X_prev = zeros(N/2, 1);
for k = 1 : K
   X = TF12(:, k);
   dX = X - X_prev;
   pos_bins = find(abs(dX) > 0);
   if (length(pos_bins) > 0)
       norm = sum(abs(X_prev).^2);
       if (norm ~= 0)
           DM12(k) = log(sum(abs(dX(pos_bins)).^2) / sum(abs(X_prev).^2));
       end
   end
   X_prev = X;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Thresholding
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% keeping only positive values
JE5 = JE5 .* (JE5 > 0);
JE4 = JE4 .* (JE4 > 0);
JE3 = JE3 .* (JE3 > 0);
JE2 = JE2 .* (JE2 > 0);
DM12 = DM12 .* (DM12 > 0);

% histograms
[hist5, je5] = hist(JE5, round(length(JE5)/5));
[hist4, je4] = hist(JE4, round(length(JE4)/5));
[hist3, je3] = hist(JE3, round(length(JE3)/5));
[hist2, je2] = hist(JE2, round(length(JE2)/5));
[hist12, je12] = hist(DM12, round(length(DM12)/5));
hist5 = hist5 ./ sum(hist5);
hist4 = hist4 ./ sum(hist4);
hist3 = hist3 ./ sum(hist3);
hist2 = hist2 ./ sum(hist2);
hist12 = hist12 ./ sum(hist12);

% probability distribution
cs5 = cumsum(hist5);
cs4 = cumsum(hist4);
cs3 = cumsum(hist3);
cs2 = cumsum(hist2);
cs12 = cumsum(hist12);

% simplified threshold calculation
if nargin == 2
   epsilon = 1 - sensitivity;
else
   epsilon = 1 - 0.01;    
end
thr5 = je5(min(find(cs5 >= epsilon)));
thr4 = je4(min(find(cs4 >= epsilon)));
thr3 = je3(min(find(cs3 >= epsilon)));
thr2 = je2(min(find(cs2 >= epsilon)));
thr12 = je12(min(find(cs12 >= epsilon)));

% thresholding
OS5 = find(JE5 >= thr5);
OS4 = find(JE4 >= thr4);
OS3 = find(JE3 >= thr3);
OS2 = find(JE2 >= thr2);
OS12 = find(DM12 >= thr12);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Combining subband information
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% detection functions
detect5 = zeros(size(signal));
detect4 = zeros(size(signal));
detect3 = zeros(size(signal));
detect2 = zeros(size(signal));
detect12 = zeros(size(signal));
detect5(OS5*N/2) = 1;
detect4(OS4*N) = 1;
detect3(OS3*N*2) = 1;
detect2(OS2*N*4) = 1;
detect12(OS12*N*4) = 1;

% weighting the output of each subband
detect = 0.5 * detect12 + 0.6 * detect2 + 0.8 * detect3 + detect4 + detect5;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Windowing
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% length of the smoothing time-window (in samples)
ds = 4096;

% windowing
detect_temp = detect;
os = [];
stop = 0;
while (stop == 0)
   [val, pos] = max(detect_temp);
   tmin = max(pos - ds/2, 1);
   tmax = min(pos + ds/2, L);
   detect_temp(tmin : tmax) = 0;
   os = [os, pos];
   if (length(find(detect_temp ~= 0)) == 0)
       stop = 1;
   end
end
onset = sort(os);
